<?php

class ITSEC_Malware_Scan_Results_Template {
	public static function get_html( $results, $show_error_details = false ) {
		$html = "<div class='itsec-malware-scan-results'>\n";
		
		if ( is_wp_error( $results ) ) {
			$html .= self::get_wp_error_details( $results, $show_error_details );
		} else {
			$html .= self::get_system_error_details( $results );
			$html .= self::get_malware_details( $results );
			$html .= self::get_blacklist_details( $results );
		}
		
		$html .= "</div>\n";
		
		return $html;
	}
	
	protected static function get_wp_error_details( $results, $show_error_details ) {
		$status = 'error';
		$description = __( 'The scan failed to properly scan the site.', 'it-l10n-ithemes-security-pro' );
		
		$details = '<p>' . sprintf( __( 'Error Message: %s', 'it-l10n-ithemes-security-pro' ), $results->get_error_message() ) . "</p>\n";
		$details .= '<p>' . sprintf( __( 'Error Code: <code>%s</code>', 'it-l10n-ithemes-security-pro' ), esc_html( $results->get_error_code() ) ) . "</p>\n";
		
		if ( $show_error_details ) {
			$data = $results->get_error_data();
			
			if ( ! empty( $data ) ) {
				$details .= '<p>' . __( 'If you contact support about this error, please provide the following debug details:', 'it-l10n-ithemes-security-pro' ) . "</p>\n";
				$details .= '<pre>' . print_r( $data, true ) . "</pre>\n";
			}
		}
		
		return self::get_wrapped_section( 'wp-error', $status, $description, $details );
	}
	
	protected static function get_blacklist_details( $results ) {
		$status = 'clean';
		$description = __( 'Blacklist', 'it-l10n-ithemes-security-pro' );
		$details = false;
		
		$list = '';
		
		if ( ! empty( $results['BLACKLIST']['WARN'] ) ) {
			$status = 'warn';
			
			foreach ( (array) $results['BLACKLIST']['WARN'] as $entry ) {
				$list .= sprintf( '<li class="itsec-malware-scan-warn"><span><a href="%1$s">%2$s</a><span></li>', $entry[1], $entry[0] ) . "\n";
			}
		}
		
		if ( ! empty( $results['BLACKLIST']['INFO'] ) ) {
			foreach ( (array) $results['BLACKLIST']['INFO'] as $entry ) {
				$list .= sprintf( '<li class="itsec-malware-scan-clean"><span><a href="%1$s">%2$s</a></span></li>', $entry[1], $entry[0] ) . "\n";
			}
		}
		
		if ( ! empty( $list ) ) {
			$details = "<ul>\n$list</ul>\n";
		}
		
		return self::get_wrapped_section( 'malware', $status, $description, $details );
	}
	
	protected static function get_malware_details( $results ) {
		$status = 'clean';
		$description = __( 'Malware', 'it-l10n-ithemes-security-pro' );
		$details = false;
		
		$list = '';
		
		if ( ! empty( $results['MALWARE']['WARN'] ) ) {
			$status = 'warn';
			
			foreach ( (array) $results['MALWARE']['WARN'] as $entry ) {
				list( $message, $url ) = explode( ':', $entry[0], 2 );
				$message = trim( $message );
				$url = trim( $url );
				
				$message_lowercase = strtolower( $message );
				
				if ( 'security warning in the url' === $message_lowercase ) {
					$message = __( 'Security warning in the URL', 'it-l10n-ithemes-security-pro' );
				} else if ( 'malware found on url' === $message_lowercase ) {
					$message = __( 'Malware found on URL', 'it-l10n-ithemes-security-pro' );
				}
				
				if ( '&amp;' === substr( $url, 0, 5 ) ) {
					$payload = html_entity_decode( $url );
					$url = get_site_url();
				}
				
				$message .= "<br>\n";
				$message .= sprintf( __( 'Infected URL: <a href="%1$s" target="_blank">%2$s</a>', 'it-l10n-ithemes-security-pro' ), esc_url( $url ), esc_html( $url ) );
				
				$parts = explode( "\n", $entry[1], 2 );
				
				if ( isset( $parts[1] ) ) {
					if ( preg_match( '/(.+)\. Details: (.+)/', $parts[0], $match ) ) {
						$type = $match[1];
						$documentation_url = $match[2];
						
						if ( '*Known Spam detected' === $type ) {
							$type = __( '*Known Spam detected', 'it-l10n-ithemes-security-pro' );
						}
						
						$message .= "<br>\n";
						$message .= sprintf( __( 'Type: %1$s', 'it-l10n-ithemes-security-pro' ), $type );
						$message .= "<br>\n";
						$message .= sprintf( __( 'Documentation: <a href="%1$s" target="_blank">%2$s</a>', 'it-l10n-ithemes-security-pro' ), esc_url( $documentation_url ), esc_html( $documentation_url ) );
					}
					
					$payload_raw = trim( $parts[1] );
					$payload_raw = html_entity_decode( $payload_raw );
					
					if ( preg_match( '/<div id=\'HiddenDiv\'>(.+)<\/div>/', $payload_raw, $match ) ) {
						$payload = trim( $match[1] );
					}
				}
				
				if ( ! empty( $payload ) ) {
					$message .= "<br>\n";
					$message .= sprintf( __( 'Payload:<pre>%s</pre>', 'it-l10n-ithemes-security-pro' ), esc_html( $payload ) );
				}
				
				$list .= sprintf( '<li class="itsec-malware-scan-warn"><span>%s</span></li>', $message ) . "\n";
			}
		}
		
		if ( ! empty( $list ) ) {
			$details = "<ul>\n$list</ul>\n";
		}
		
		return self::get_wrapped_section( 'malware', $status, $description, $details );
	}
	
	protected static function get_system_error_details( $results ) {
		if ( empty( $results['SYSTEM']['ERROR'] ) ) {
			return '';
		}
		
		$status = 'error';
		$description = __( 'The scan failed to properly scan the site.', 'it-l10n-ithemes-security-pro' );
		
		$details = "<ul>\n";
		
		foreach ( $results['SYSTEM']['ERROR'] as $entry ) {
			$details .= sprintf( '<li class="itsec-malware-scan-error"><span>%s</span></li>', $entry ) . "\n";
		}
		
		$details .= "</ul>\n";
		
		return self::get_wrapped_section( 'system-error', $status, $description, $details );
	}
	
	protected static function get_wrapped_section( $type, $status, $description, $details = false ) {
		$html = "<div class='itsec-malware-scan-results-section itsec-malware-scan-results-$type-section'>\n";
		
		if ( 'clean' === $status ) {
			$status_text = __( 'Clean', 'it-l10n-ithemes-security-pro' );
		} else if ( 'warn' === $status ) {
			$status_text = __( 'Warning', 'it-l10n-ithemes-security-pro' );
		} else if ( 'error' === $status ) {
			$status_text = __( 'Error', 'it-l10n-ithemes-security-pro' );
		}
		
		$status = "<span class='itsec-malware-scan-$status'>$status_text</span>";
		
		if ( false === $details ) {
			/* translators: Scan result listing. %1$s is the status. %2$s is the description. */
			$html .= "<p>" . sprintf( _x( '%1$s %2$s', 'scan status, scan description', 'it-l10n-ithemes-security-pro' ), $status, $description ) . "</p>\n";
		} else {
			$details_link = "<a href='#' class='itsec-malware-scan-toggle-details'>" . __( 'Show Details', 'it-l10n-ithemes-security-pro' ) . "</a>\n";
			
			/* translators: Scan result listing. %1$s is the status. %2$s is the description. %3$s is the details link. */
			$html .= "<p>" . sprintf( _x( '%1$s %2$s %3$s', 'scan status, scan description, scan details link', 'it-l10n-ithemes-security-pro' ), $status, $description, $details_link ) . "</p>\n";
			
			$html .= "<div class='itsec-malware-scan-details'>\n$details\n</div>\n";
		}
		
		$html .= "</div>\n";
		
		return $html;
	}
}
